@@ -3,6 +3,7 @@ import urllib.parse
 import os.path
 import re
 import pathlib
+from io import BytesIO
 from sys import platform
 
 from .conf import settings
@@ -46,6 +47,9 @@ class Video(object):
         self.playback_info = None
         self.media_source = None
         self.srcid = srcid
+        self.intro_tried = False
+        self.intro_start = None
+        self.intro_end = None
 
     def map_streams(self):
         self.subtitle_seq = {}
@@ -147,9 +151,12 @@ class Video(object):
 
     def terminate_transcode(self):
         if self.is_transcode:
-            self.client.jellyfin.close_transcode(
-                self.client.config.data["app.device_id"]
-            )
+            try:
+                self.client.jellyfin.close_transcode(
+                    self.client.config.data["app.device_id"]
+                )
+            except:
+                log.warning("Terminating transcode failed.", exc_info=1)
 
     def _get_url_from_source(self):
         # Only use Direct Paths if:
@@ -227,6 +234,66 @@ class Video(object):
                 log.warning("Preferred media source is unplayable.")
             return selected
 
+    def get_intro(self, media_source_id):
+        if self.intro_tried:
+            return
+        self.intro_tried = True
+
+        # provided by plugin
+        try:
+            skip_intro_data = self.client.jellyfin._get(
+                f"Episode/{media_source_id}/IntroTimestamps"
+            )
+            if skip_intro_data is not None and skip_intro_data["Valid"]:
+                self.intro_start = skip_intro_data["IntroStart"]
+                self.intro_end = skip_intro_data["IntroEnd"]
+        except:
+            log.warning(
+                "Fetching intro data failed. Do you have the plugin installed?",
+                exc_info=1,
+            )
+
+    def get_chapters(self):
+        return [
+            {"start": item["StartPositionTicks"] / 10000000, "name": item["Name"]}
+            for item in self.item.get("Chapters", [])
+        ]
+
+    def get_chapter_images(self, max_width=400, quality=90):
+        for i, item in enumerate(self.item.get("Chapters", [])):
+            data = BytesIO()
+            self.client.jellyfin._get_stream(
+                f"Items/{self.item_id}/Images/Chapter/{i}",
+                data,
+                {"tag": item["ImageTag"], "maxWidth": max_width, "quality": quality},
+            )
+            yield data.getvalue()
+
+    def get_bif(self, prefer_width=320):
+        # requires JellyScrub plugin
+        data = BytesIO()
+        manifest = self.client.jellyfin._get(
+            f"Trickplay/{self.media_source['Id']}/GetManifest"
+        )
+        if (
+            manifest is not None
+            and manifest.get("WidthResolutions") is not None
+            and len(manifest["WidthResolutions"]) > 0
+        ):
+            if prefer_width is not None:
+                width = min(
+                    manifest["WidthResolutions"], key=lambda x: abs(x - prefer_width)
+                )
+            else:
+                width = manifest["WidthResolutions"][-1]
+            self.client.jellyfin._get_stream(
+                f"Trickplay/{self.media_source['Id']}/{width}/GetBIF", data
+            )
+        else:
+            return None
+        data.seek(0)
+        return data
+
     def get_playback_url(
         self,
         video_bitrate: Optional[int] = None,
@@ -251,6 +318,9 @@ class Video(object):
         )
 
         self.media_source = self.get_best_media_source(self.srcid)
+        if settings.skip_intro_always or settings.skip_intro_prompt:
+            self.get_intro(self.media_source["Id"])
+
         self.map_streams()
         url = self._get_url_from_source()
 
@@ -379,7 +449,7 @@ class Media(object):
 
     def replace_queue(self, sp_items, seq):
         """Update queue for SyncPlay.
-           Returns None if the video is the same or a new Media if not."""
+        Returns None if the video is the same or a new Media if not."""
         if self.queue[self.seq]["Id"] == sp_items[seq]["Id"]:
             self.queue, self.seq = sp_items, seq
             return None
